//------------------------------------------------------------------------------
//
// Copyright (c) 2004-2005  Matt Brunk  <info@smartcache.net>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License VERSION 2 as
// published by the Free Software Foundation.  You are not allowed to
// use any other version of the license; unless you got the explicit
// permission from the author to do so.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//------------------------------------------------------------------------------
#include "stdafx.h"
#include <ctype.h>
#include "HexEdit.h"

#define BLANK_ENTRY		_T("")

const UINT CHexEdit::s_nDefaultMsg = ::RegisterWindowMessage(_T("CHexEdit_DefaultMsg"));

/////////////////////////////////////////////////////////////////////////////
// CHexEdit

CHexEdit::CHexEdit()
{
	m_lastValidValue	= _T("");
	m_bDoingUpdate		= false;
	m_hwndDest			= NULL;
	m_bAllowSpaces		= false;
}

CHexEdit::~CHexEdit()
{
}	//lint !e1540: Pointer member 'CHexEdit::m_hwndDest' neither freed nor zeroed by destructor


BEGIN_MESSAGE_MAP(CHexEdit, CEdit)
	//{{AFX_MSG_MAP(CHexEdit)
	ON_CONTROL_REFLECT(EN_UPDATE, OnUpdate)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHexEdit message handlers

void CHexEdit::SetOwner(HWND hWnd)
{
	m_hwndDest = hWnd;
}

void CHexEdit::SetAllowSpaces(bool bAllow)
{
	m_bAllowSpaces = bAllow;
}



void CHexEdit::OnUpdate() 
{
	CString strWindowText;

	//--------------------------------------------------------------------------
	//	If we're doing an update, return.  Otherwise we'll recurse forever and 
	//	a stack overflow will occur.
	//--------------------------------------------------------------------------
	if ( m_bDoingUpdate )
	{
		return;
	}

	//--------------------------------------------------------------------------
	//	Copy the contents of the edit control to a string
	//--------------------------------------------------------------------------
	GetWindowText(strWindowText);

	//--------------------------------------------------------------------------
	//	Check if the edit control is blank
	//--------------------------------------------------------------------------
	if ( strWindowText.IsEmpty() )
	{
		//----------------------------------------------------------------------
		//	The operator erased the last number.  Thus the last valid value 
		//	entered by the operator is NULL.
		//----------------------------------------------------------------------
		m_lastValidValue = BLANK_ENTRY;
	}
	else
	{
		CString strUpper = strWindowText;

		strUpper.MakeUpper();

		BOOL	valid = TRUE;
		for ( int ii = 0; ii < strUpper.GetLength(); ii++ )
		{
			if ( !_istxdigit(strUpper[ii]) && (m_bAllowSpaces && !_istspace(strUpper[ii])) )
			{
				valid = FALSE;
				break;
			}
		}

		//----------------------------------------------------------------------
		//	Check if the number falls within the specified range
		//----------------------------------------------------------------------
		if ( !valid )
		{
			strWindowText = m_lastValidValue;

			//------------------------------------------------------------------
			//	Set the Edit control with the Last Valid Value
			//------------------------------------------------------------------
			SetWindowText(strWindowText);

			//------------------------------------------------------------------
			//	Place the cursor at the end of the string
			//------------------------------------------------------------------
			SetSel(strWindowText.GetLength(), strWindowText.GetLength());

			//------------------------------------------------------------------
			//	Alert the operator of the error
			//------------------------------------------------------------------
			(void)MessageBeep ( 0xFFFFFFFF );
		}
		else if ( strUpper != strWindowText )
		{
			m_bDoingUpdate = true;
			SetWindowText(strUpper);
			//------------------------------------------------------------------
			//	Place the cursor at the end of the string
			//------------------------------------------------------------------
			SetSel(strWindowText.GetLength(), strWindowText.GetLength());
			m_bDoingUpdate = false;			
		}
		else
		{
			m_lastValidValue = strWindowText;
		}
	}
	
	if ( m_hwndDest )
	{
		::PostMessage(m_hwndDest, s_nDefaultMsg, 0, GetDlgCtrlID());
	}
}

BOOL CHexEdit::PreTranslateMessage(MSG* pMsg) 
{
	//--------------------------------------------------------------------------
	//	Convert lower case hex to upper case hex.
	//--------------------------------------------------------------------------
    if ( pMsg->message == WM_CHAR )
	{
		if ( (pMsg->wParam >= 'a') && (pMsg->wParam <= 'f') )
		{
			pMsg->wParam = toupper(pMsg->wParam);
		}
	}
	
	return CEdit::PreTranslateMessage(pMsg);
}
