//------------------------------------------------------------------------------
//
// Copyright (c) 2004-2005  Matt Brunk  <info@smartcache.net>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License VERSION 2 as
// published by the Free Software Foundation.  You are not allowed to
// use any other version of the license; unless you got the explicit
// permission from the author to do so.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//	IntEdit.cpp : implementation file
//------------------------------------------------------------------------------

#include "stdafx.h"
#include <cerrno>
#include "IntEdit.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

const UINT CIntEdit::s_nDefaultMsg = ::RegisterWindowMessage(_T("CIntEdit_DefaultMsg"));

BEGIN_MESSAGE_MAP(CIntEdit, CEdit)
	//{{AFX_MSG_MAP(CIntEdit)
	ON_CONTROL_REFLECT(EN_UPDATE, OnUpdate)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CIntEdit::CIntEdit()
:
	m_szLastValidValue(_T("")),
	m_iValue(0),
	m_iMin(INT_MIN),
	m_iMax(INT_MAX),
	m_bDoingUpdate(FALSE),
	m_hwndDest(NULL)
{
}

CIntEdit::~CIntEdit()
{
}	//lint !e1540: Pointer member 'CIntEdit::m_hwndDest' neither freed nor zeroed by destructor

void CIntEdit::SetMin(int dMin)
{
	m_iMin = dMin;
}

void CIntEdit::SetMax(int dMax)
{
	m_iMax = dMax;
}

void CIntEdit::SetRange(int dMin, int dMax)
{
	m_iMin = dMin;
	m_iMax = dMax;
}

int CIntEdit::GetValue()
{
	return m_iValue;
}

void CIntEdit::SetValue(int value)
{
	CString s;

	s.Format(_T("%d"), value);
	SetWindowText(s);
}

void CIntEdit::SetOwner(HWND hWnd)
{
	m_hwndDest = hWnd;
}

bool CIntEdit::IsEmpty()
{
	CString s;
	GetWindowText(s);
	return s.IsEmpty() != 0;
}

void CIntEdit::OnUpdate() 
{
	CString	str;

	//--------------------------------------------------------------------------
	//	If we're doing an update, return.  Otherwise we'll recurse forever and 
	//	a stack overflow will occur.
	//--------------------------------------------------------------------------
	if ( m_bDoingUpdate )
	{
		return;
	}

	//--------------------------------------------------------------------------
	//	Get the text.
	//--------------------------------------------------------------------------
	GetWindowText(str);

	//--------------------------------------------------------------------------
	//	Check if the edit control is blank.
	//--------------------------------------------------------------------------
	if ( str.IsEmpty() )
	{
		//----------------------------------------------------------------------
		// No text. Remember this.
		//----------------------------------------------------------------------
		m_iValue = 0;
		m_szLastValidValue = _T("");
	}
	else
	{
		bool	valid;

		//----------------------------------------------------------------------
		//	Allow '-' and '+' as the first character.
		//----------------------------------------------------------------------
		if ( (str.GetLength() == 1) && (str.FindOneOf(_T("-+")) != -1) )
		{
			if ( str[0] == _T('-') && m_iMin >= 0 )
			{
				valid = false;
			}
			else if ( str[0] == _T('+') && m_iMax <= 0 )
			{
				valid = false;
			}
			else
			{
				m_iValue = 0;
				valid    = true;
			}
		}
		else
		{

			//----------------------------------------------------------------------
			//	Convert the string to a long using _tcstol().
			//----------------------------------------------------------------------
			TCHAR	*endPtr;
			long	x;

			errno = 0;
			x = _tcstol(str, &endPtr, 10);
			if (errno == ERANGE)
			{
				valid = false;
			}
			else if ((unsigned)(endPtr - str.GetBuffer(1)) != (unsigned)str.GetLength())
			{
				valid = false;
			}
			else if ( x < m_iMin || x > m_iMax )
			{
				valid = false;
			}
			else
			{
				m_iValue = x;
				valid    = true;
			}
		}

		//----------------------------------------------------------------------
		//	If invalid, restore the last valid value.
		//----------------------------------------------------------------------
		if ( !valid )
		{
			str = m_szLastValidValue;

			//------------------------------------------------------------------
			//	Set the Edit control with the Last Valid Value
			//------------------------------------------------------------------
			m_bDoingUpdate = true;
			SetWindowText(str);
			m_bDoingUpdate = false;

			//------------------------------------------------------------------
			//	Place the cursor at the end of the string
			//------------------------------------------------------------------
			SetSel(str.GetLength(), str.GetLength());

			//------------------------------------------------------------------
			//	Alert the operator of the error
			//------------------------------------------------------------------
			MessageBeep(0xFFFFFFFF);
		}
		else
		{
			//------------------------------------------------------------------
			//	The number is valid.  Save this as the last valid value.
			//------------------------------------------------------------------
			m_szLastValidValue = str;
		}
	}	

	if ( m_hwndDest )
	{
		::PostMessage(m_hwndDest, s_nDefaultMsg, 0, GetDlgCtrlID());
	}

}
