//------------------------------------------------------------------------------
//
// Copyright (c) 2004-2005  Matt Brunk  <info@smartcache.net>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License VERSION 2 as
// published by the Free Software Foundation.  You are not allowed to
// use any other version of the license; unless you got the explicit
// permission from the author to do so.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//------------------------------------------------------------------------------
// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "SoftCard.h"

#include "MainFrm.h"
#include "Utils.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static const CString s_szOutsideFrame			= "OutsideFrame";
static const CString s_szMainWindowPlacement		=	_T("MainWindowPlacement");

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_CLOSE()
	ON_NOTIFY(TBN_DROPDOWN, AFX_IDW_TOOLBAR, OnToolbarDropDown)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	//ID_INDICATOR_SPACE,
	ID_INDICATOR_CONNECTION,
	ID_INDICATOR_COM,
	ID_INDICATOR_BAUD
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	// TODO: add member initialization code here
	
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	DWORD dwStyle;
	dwStyle = m_wndToolBar.GetButtonStyle(m_wndToolBar.CommandToIndex(IDM_CONNECT_RS232));
	dwStyle |= TBSTYLE_DROPDOWN;
	m_wndToolBar.SetButtonStyle(m_wndToolBar.CommandToIndex(IDM_CONNECT_RS232), dwStyle);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers


void CMainFrame::ActivateFrame(int nCmdShow) 
{
	// nCmdShow is the normal show mode this frame should be in
	// translate default nCmdShow (-1)
	if (nCmdShow == -1)
	{
		if (!IsWindowVisible())
			nCmdShow = SW_SHOWNORMAL;
		else if (IsIconic())
			nCmdShow = SW_RESTORE;
	}

	// bring to top before showing
	BringToTop(nCmdShow);

	if (nCmdShow != -1)
	{
		// show the window as specified
	  WINDOWPLACEMENT wp;
	  
	  if ( !CUtils::ReadWindowPlacement(s_szOutsideFrame, s_szMainWindowPlacement, &wp) )
	  {
		  ShowWindow(nCmdShow);
	  }
	  else
	  {
		if ( nCmdShow != SW_SHOWNORMAL )  
		  wp.showCmd = nCmdShow;

		SetWindowPlacement(&wp);
	   // ShowWindow(wp.showCmd);
	  }

	  // and finally, bring to top after showing
	  BringToTop(nCmdShow);
	}
	
	//CFrameWnd::ActivateFrame(nCmdShow);
}

void CMainFrame::OnClose() 
{
	WINDOWPLACEMENT	wp;

	// before it is destroyed, save the position of the window
	wp.length = sizeof wp;

	if ( GetWindowPlacement(&wp) )
	{

		if ( IsIconic() )
		  // never restore to Iconic state
		  wp.showCmd = SW_SHOW ;

		if ((wp.flags & WPF_RESTORETOMAXIMIZED) != 0)
		  // if maximized and maybe iconic restore maximized state
		  wp.showCmd = SW_SHOWMAXIMIZED ;

		// and write it to the .INI file
		CUtils::WriteWindowPlacement(s_szOutsideFrame, s_szMainWindowPlacement, &wp);
	}

	CFrameWnd::OnClose();
}

void CMainFrame::OnToolbarDropDown(NMTOOLBAR* pnmtb, LRESULT* /*plr*/)
{
	CWnd	*pWnd   = NULL;
	CMenu	*pPopup = NULL;

	// Switch on button command id's.
	switch (pnmtb->iItem)
	{
		case IDM_CONNECT_RS232:
		{
			pWnd = &m_wndToolBar;
			pPopup = GetConnectMenu("Port");
		}
		break;

		default:
			return;
	}

	if ( !pPopup )
	{
		return;
	}

	CRect rc;
	pWnd->SendMessage(TB_GETRECT, pnmtb->iItem, (LPARAM)&rc);
	pWnd->ClientToScreen(&rc);
	
	pPopup->TrackPopupMenu
	(
		TPM_LEFTALIGN | TPM_LEFTBUTTON | TPM_VERTICAL,
		rc.left,
		rc.bottom,
		this,
		&rc
	);
}

CMenu* CMainFrame::GetConnectMenu(LPCTSTR /*popupName*/)
{
	CMenu *pMenu = GetMenu();
	ASSERT(pMenu);

	//--------------------------------------------------------------------------
	//	Loop through menu names.
	//--------------------------------------------------------------------------
	for ( UINT iPos = 0; iPos < pMenu->GetMenuItemCount(); iPos++ )
	{
		CString name;
		pMenu->GetMenuString(iPos, name, MF_BYPOSITION);
		if ( name.Find(_T("Connect")) >= 0 )
		{
			CMenu* pSubMenu = pMenu->GetSubMenu(iPos);
			return pSubMenu;
		}
	}

	return NULL;
}

void CMainFrame::SetCom(int iPort, int iBaud)
{
	CString	szInterface = _T(" ");
	CString	szBaud;

	if ( iPort > 0 )
	{
		szInterface.Format("COM%d", iPort);
	}

	szBaud.Format("%d", iBaud);
	SetCom(szInterface, szBaud);
}

void CMainFrame::SetCom(LPCTSTR szInterface, LPCTSTR other)
{
	UINT	nID;
	int		nIndex;
	UINT	nStyle;
	int		nWidth;
	CString sss;
	CSize	szExtent;

	CClientDC dc(&m_wndStatusBar);
	CFont* pOldFont = dc.SelectObject(m_wndStatusBar.GetFont());

	sss = szInterface;
	nIndex = m_wndStatusBar.CommandToIndex(ID_INDICATOR_COM);
	szExtent = dc.GetTextExtent(sss, _tcslen(sss));
	m_wndStatusBar.GetPaneInfo(nIndex, nID, nStyle, nWidth);
	m_wndStatusBar.SetPaneInfo(nIndex, nID, nStyle, szExtent.cx);
	m_wndStatusBar.SetPaneText(nIndex, sss);


	sss = other;
	nIndex = m_wndStatusBar.CommandToIndex(ID_INDICATOR_BAUD);
	szExtent = dc.GetTextExtent(sss, _tcslen(sss));
	m_wndStatusBar.GetPaneInfo(nIndex, nID, nStyle, nWidth);
	m_wndStatusBar.SetPaneInfo(nIndex, nID, nStyle, szExtent.cx);
	m_wndStatusBar.SetPaneText(nIndex, sss);

	dc.SelectObject(pOldFont);

}


void CMainFrame::SetConnection(LPCTSTR name)
{
	UINT	nID;
	UINT	nStyle;
	int		nWidth;

	int nIndex = m_wndStatusBar.CommandToIndex(ID_INDICATOR_CONNECTION);

	CClientDC dc(&m_wndStatusBar);
	CFont* pOldFont = dc.SelectObject(m_wndStatusBar.GetFont());
	CSize szExtent = dc.GetTextExtent(name, _tcslen(name));
	dc.SelectObject(pOldFont);

	m_wndStatusBar.GetPaneInfo(nIndex, nID, nStyle, nWidth);
	m_wndStatusBar.SetPaneInfo(nIndex, nID, nStyle, szExtent.cx);

	m_wndStatusBar.SetPaneText(nIndex, name);
}


void CMainFrame::SetBaud(int iBaud)
{
	UINT	nID;
	int		nIndex;
	UINT	nStyle;
	int		nWidth;
	CString sss;
	CSize	szExtent;

	CClientDC dc(&m_wndStatusBar);
	CFont* pOldFont = dc.SelectObject(m_wndStatusBar.GetFont());

	sss.Format(_T("%d"), iBaud);
	nIndex = m_wndStatusBar.CommandToIndex(ID_INDICATOR_BAUD);
	szExtent = dc.GetTextExtent(sss, _tcslen(sss));
	m_wndStatusBar.GetPaneInfo(nIndex, nID, nStyle, nWidth);
	m_wndStatusBar.SetPaneInfo(nIndex, nID, nStyle, szExtent.cx);
	m_wndStatusBar.SetPaneText(nIndex, sss);

	dc.SelectObject(pOldFont);

}

