//------------------------------------------------------------------------------
//
// Copyright (c) 2004-2005  Matt Brunk  <info@smartcache.net>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License VERSION 2 as
// published by the Free Software Foundation.  You are not allowed to
// use any other version of the license; unless you got the explicit
// permission from the author to do so.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//------------------------------------------------------------------------------
// SoftCardView.cpp : implementation of the CSoftCardView class
//

#include "stdafx.h"
#include "SoftCard.h"
#include "SoftCardTypes.h"

#include "SoftCardDoc.h"
#include "SoftCardView.h"
#include "MainFrm.h"
#include "MyTime.h"
#include "EnumPorts.h"
#include "DlgATR.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static const COLORREF	COLOR_BLACK				= RGB(0x00, 0x00, 0x00);
static const COLORREF	COLOR_CMD				= RGB(0x00, 0x00, 0xFF);
static const COLORREF	COLOR_CMD_NAME			= RGB(0x80, 0x00, 0x00);
static const COLORREF	COLOR_PROCEDURE_BYTE	= RGB(0x00, 0x80, 0x00);
static const COLORREF	COLOR_DATA				= COLOR_BLACK;
static const COLORREF	COLOR_STATUS			= RGB(0x00, 0x80, 0x00);
static const COLORREF	COLOR_NULL_BYTE			= RGB(0x80, 0x80, 0x80);
static const COLORREF	COLOR_HEADER			= RGB(0xFF, 0x00, 0x00);
static const int		IDM_CONNECT_RS232_BASE	= 62000;
static const int		DEFAULT_BAUD_RATE		= 9600;


static const int		TIMER_ATR				= 0x657;
static const UINT		TIMER_ATR_DURATION		= 200;		// ms

static const CString s_szSectionCom			= "COM";
static const CString s_szBaud				=	"Baud";

static const CString s_szSectionATR			= "ATR";
static const CString s_szIsAcos1				=	"IsAcos1";
static const CString s_szAcos1ATR				=	"Acos1ATR";
static const CString s_szOtherATR				=	"OtherATR";

static CString GetCmdString(U8 ins);

/////////////////////////////////////////////////////////////////////////////
// CSoftCardView

IMPLEMENT_DYNCREATE(CSoftCardView, CRichEditView)

BEGIN_MESSAGE_MAP(CSoftCardView, CRichEditView)
	//{{AFX_MSG_MAP(CSoftCardView)
	ON_WM_CHAR()
	ON_WM_TIMER()
	ON_COMMAND(IDM_DISCONNECT, OnDisconnect)
	ON_UPDATE_COMMAND_UI(IDM_DISCONNECT, OnUpdateDisconnect)
	ON_COMMAND(IDM_CLEAR, OnClear)
	ON_UPDATE_COMMAND_UI(IDM_CLEAR, OnUpdateClear)
	ON_COMMAND(IDM_GOTO_TOP, OnGotoTop)
	ON_UPDATE_COMMAND_UI(IDM_GOTO_TOP, OnUpdateGotoTop)
	ON_UPDATE_COMMAND_UI(IDM_CONNECT_RS232, OnUpdateConnectRs232)
	ON_COMMAND(IDM_BAUD_9600, OnBaud9600)
	ON_UPDATE_COMMAND_UI(IDM_BAUD_9600, OnUpdateBaud9600)
	ON_COMMAND(IDM_BAUD_19200, OnBaud19200)
	ON_UPDATE_COMMAND_UI(IDM_BAUD_19200, OnUpdateBaud19200)
	ON_COMMAND(IDM_BAUD_38400, OnBaud38400)
	ON_UPDATE_COMMAND_UI(IDM_BAUD_38400, OnUpdateBaud38400)
	ON_COMMAND(IDM_BAUD_57600, OnBaud57600)
	ON_UPDATE_COMMAND_UI(IDM_BAUD_57600, OnUpdateBaud57600)
	ON_COMMAND(IDM_BAUD_115200, OnBaud115200)
	ON_UPDATE_COMMAND_UI(IDM_BAUD_115200, OnUpdateBaud115200)
	ON_COMMAND(IDM_BAUD_128000, OnBaud128000)
	ON_UPDATE_COMMAND_UI(IDM_BAUD_128000, OnUpdateBaud128000)
	ON_COMMAND(IDM_BAUD_256000, OnBaud256000)
	ON_UPDATE_COMMAND_UI(IDM_BAUD_256000, OnUpdateBaud256000)
	ON_COMMAND(IDM_ATR, OnAtr)
	ON_UPDATE_COMMAND_UI(IDM_ATR, OnUpdateAtr)
	ON_COMMAND(IDM_CARD_NEW, OnCardNew)
	ON_UPDATE_COMMAND_UI(IDM_CARD_NEW, OnUpdateCardNew)
	ON_COMMAND(IDM_CARD_LOAD, OnCardLoad)
	ON_UPDATE_COMMAND_UI(IDM_CARD_LOAD, OnUpdateCardLoad)
	ON_COMMAND(IDM_CLEAR_ON_RESET, OnClearOnReset)
	ON_UPDATE_COMMAND_UI(IDM_CLEAR_ON_RESET, OnUpdateClearOnReset)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CRichEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CRichEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CRichEditView::OnFilePrintPreview)
	ON_WM_SERIAL(OnSerialMsg)	//lint !e1773: Attempt to cast away const (or volatile)
	ON_COMMAND_RANGE(IDM_CONNECT_RS232_BASE, (IDM_CONNECT_RS232_BASE + 255), OnConnectRS232)	//lint !e1773 Attempt to cast away const
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSoftCardView construction/destruction

CSoftCardView::CSoftCardView()
{
	// TODO: add construction code here

	m_iLineLength		= 0;
	m_iPort				= 1;
	m_iBaud				= 9600;
	m_eCardState		= STATE_ATR;
	m_iIdx				= 0;
	m_bASCII			= false;
	m_bInitCalled		= false;
	::ZeroMemory(m_buf, sizeof m_buf);
	m_pMenu				= NULL;
	m_lastReceiveTime	= MyTime::GetTime();
	m_iNumWrite			= 0;
	m_bClearOnReset		= TRUE;
	::ZeroMemory(&m_atr, sizeof m_atr);
	::ZeroMemory(m_cardBuf, sizeof m_cardBuf);
	m_cardBufLength		= 0;
}

CSoftCardView::~CSoftCardView()
{

	try
	{
		if ( m_bInitCalled )
		{
			WriteSettings();
		}
		if ( m_serial.IsOpen() )
		{
			m_serial.Close();
		}
	}
	catch (...)
	{

	}

}	//lint !e1740: pointer member m_pMenu not directly freed or zeroed by destructor

BOOL CSoftCardView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CRichEditView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CSoftCardView drawing

void CSoftCardView::OnDraw(CDC* pDC)
{
	//CSoftCardDoc* pDoc = GetDocument();
	//ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
	CRichEditView::OnDraw(pDC);
}

/////////////////////////////////////////////////////////////////////////////
// CSoftCardView printing

BOOL CSoftCardView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	//return DoPreparePrinting(pInfo);
	return CRichEditView::DoPreparePrinting(pInfo);

}

void CSoftCardView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	CRichEditView::OnBeginPrinting(pDC, pInfo);
	// TODO: add extra initialization before printing
}

void CSoftCardView::OnEndPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	CRichEditView::OnEndPrinting(pDC, pInfo);
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CSoftCardView diagnostics

#ifdef _DEBUG
void CSoftCardView::AssertValid() const
{
	CRichEditView::AssertValid();
}

void CSoftCardView::Dump(CDumpContext& dc) const
{
	CRichEditView::Dump(dc);
}

CSoftCardDoc* CSoftCardView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSoftCardDoc)));
	return (CSoftCardDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSoftCardView message handlers

void CSoftCardView::Init()
{
	ReadSettings();

	if ( m_atr.bAcos1 )
	{
		m_acos1.SetATR(m_atr.acos1Atr, m_atr.iAcos1AtrLength);
	}
	else
	{
		m_acos1.SetATR(m_atr.otherAtr, m_atr.iOtherAtrLength);
	}

	m_pMenu = AfxGetMainWnd()->GetMenu();

	ASSERT(m_pMenu);

	CHARFORMAT	cf;
	ZeroMemory(&cf, sizeof cf);
	cf.cbSize = sizeof cf;

	// Apply new font
	cf.yHeight = 10 * 20;
	lstrcpy(cf.szFaceName, _T("Courier New") );

	cf.dwMask		= CFM_FACE | CFM_SIZE;
	cf.dwEffects	= 0;

	GetRichEditCtrl().SetDefaultCharFormat(cf);

	CEnumPorts::GetAvailablePorts(m_ports);
	UpdateConnectMenu();

	m_iBaud = AfxGetApp()->GetProfileInt(s_szSectionCom, s_szBaud, DEFAULT_BAUD_RATE);

#if 0
	if ( (m_iBaud != DEFAULT_BAUD_RATE) && (m_iBaud != 115200) )
	{
		m_iBaud = DEFAULT_BAUD_RATE;
	}
#endif

	CMainFrame *pFrame = dynamic_cast<CMainFrame *>(AfxGetMainWnd());
	pFrame->SetConnection(_T("Disconnected"));
	pFrame->SetCom(0, m_iBaud);

	UpdateBaudMenu();
	m_bInitCalled = true;
}

LONG CSoftCardView::InitPort()
{
	CString szPort;
	szPort.Format("\\\\.\\COM%d", m_iPort);

	LONG error = m_serial.Open(szPort, this);
	if ( !error )
	{
		CMainFrame *pFrame = dynamic_cast<CMainFrame *>(AfxGetMainWnd());
		pFrame->SetCom(m_iPort, m_iBaud);
		pFrame->SetConnection(_T("Connected"));

		error = m_serial.Setup((CSerial::EBaudrate)m_iBaud, CSerial::EData8, CSerial::EParEven, CSerial::EStop2);
		ASSERT(error == 0);
		error = m_serial.SetupHandshaking(CSerial::EHandshakeOff);
		ASSERT(error == 0);
		m_serial.Flush();
	}
	else
	{
		CString msg;
		msg.Format(_T("Error opening serial port COM%d.\nThe serial port may be open by another application or driver."), m_iPort);
		AfxMessageBox(msg, MB_OK | MB_ICONERROR);
	}
	return error;
}

LRESULT CSoftCardView::OnSerialMsg(WPARAM wParam, LPARAM /*lParam*/)
{

	CSerial::EEvent eEvent = CSerial::EEvent(LOWORD(wParam));
	//CSerial::EError eError = CSerial::EError(HIWORD(wParam));
	
	if ( eEvent & CSerial::EEventRLSD )
	{
		if ( m_serial.GetRLSD() )
		{
			if ( m_bClearOnReset )
			{
				OnClear();
			}

			m_acos1.DoATR();

			if ( !AtStartOfLine() )
			{
				AddLine(_T(""), COLOR_DATA);
			}
			AddLine(_T("RESET ---------"), COLOR_HEADER);

			U8	atr[MAX_ATR_LENGTH];
			UINT	iLength;

			m_acos1.GetATR(atr, iLength);

			OutputCmdData(atr, iLength);
			AddLine(_T(""), COLOR_DATA);
			m_iNumWrite		= iLength;
			m_eCardState	= STATE_ECHO;
			m_iIdx = 0;
			m_serial.Write(atr, iLength);

			//SetTimer(TIMER_ATR, TIMER_ATR_DURATION, NULL);
		}
	}

	if (eEvent & CSerial::EEventRecv)
	{
		DWORD			dwRead;
		unsigned char	szData[2048];
		const int		nBuflen = sizeof(szData) - 1;

		do
		{
			m_serial.Read(szData, nBuflen, &dwRead);

			//TRACE("dwRead = %ld\n", dwRead);

			DisplayData(szData, dwRead);			

		} while (dwRead == nBuflen);
	}

	m_lastReceiveTime = MyTime::GetTime();

	return 0;

}

void CSoftCardView::DisplayData(unsigned char *pBuf, DWORD length)
{
	CRichEditCtrl	&edit = GetRichEditCtrl();
	int				editLength;
	CString			sss;

	for ( DWORD ii = 0; ii < length; ii++ )
	{
		//TRACE("%02X\n", (int)pBuf[ii]);
		switch ( m_eCardState )
		{
			case STATE_ECHO:
			{
				m_iNumWrite--;
				if ( m_iNumWrite == 0 )
				{
					m_eCardState = STATE_CMD;
				}
			}
			break;

			case STATE_ATR:
			{
				KillTimer(TIMER_ATR);
				SetTimer(TIMER_ATR, TIMER_ATR_DURATION, NULL);
				sss.Format(_T("%02X "), (int)pBuf[ii]);

				editLength = edit.GetWindowTextLength();
				edit.SetSel(editLength, editLength);
				edit.ReplaceSel(sss);
				edit.LineScroll( edit.GetLineCount() );
				GetDocument()->SetModifiedFlag(FALSE);
			}
			break;

			case STATE_CMD:
			{
				m_buf[m_iIdx] = pBuf[ii];
				m_iIdx++;
				if ( m_iIdx == 5 )
				{
					OutputCmd(m_buf);
					m_acos1.ProcessCommand(m_buf, m_cardBuf, m_cardBufLength);
					m_eCardState = STATE_INS;
					if ( m_cardBufLength > 0 )
					{
						m_serial.Write(m_cardBuf, m_cardBufLength);
					}
				}

			}
			break;

			case STATE_INS:
			{
				m_buf[m_iIdx] = pBuf[ii];
				m_iIdx++;
				OutputProcedureByte(m_buf[5]);
				if ( m_buf[4] > 0 )
				{
					m_eCardState = STATE_DATA;
				}
				else
				{
					m_eCardState = STATE_STATUS;
				}

			}
			break;

			case STATE_DATA:
			{
				m_buf[m_iIdx] = pBuf[ii];
				m_iIdx++;
				if ( m_iIdx == (5 + 1 + m_buf[4]) )
				{
					OutputCmdData(&m_buf[6], m_buf[4]);

					m_acos1.ProcessCommandData(&m_buf[6], m_buf[4], m_cardBuf, m_cardBufLength);
					m_eCardState = STATE_STATUS;
					if ( m_cardBufLength > 0 )
					{
						m_serial.Write(m_cardBuf, m_cardBufLength);
					}
				}
			}
			break;


			case STATE_STATUS:
			{
				if ( pBuf[ii] == 0x60 )
				{
					OutputNull();
				}
				else
				{
					m_buf[m_iIdx] = pBuf[ii];
					m_iIdx++;
					if ( m_iIdx == (5 + 1 + m_buf[4] + 2) )
					{
						OutputStatus(&m_buf[5 + 1 + m_buf[4]]);
						m_eCardState = STATE_CMD;
						m_iIdx = 0;
					}
				}
			}
			break;

			case STATE_NONE:
			default:
			{
			}
			break;
		}

		GetDocument()->SetModifiedFlag(FALSE);
	}
}

void CSoftCardView::OnChar(UINT /*nChar*/, UINT /*nRepCnt*/, UINT /*nFlags*/) 
{
	::MessageBeep(0xFFFFFFFF);
}

void CSoftCardView::OnInitialUpdate() 
{	
   // Turn on the horizontal scroll bar
   m_nWordWrap = WrapNone;
   WrapChanged();

	Init();	
	CRichEditView::OnInitialUpdate();
	//--------------------------------------------------------------------------
	//	Set the printing margins (720 twips = 1/2 inch).
	//--------------------------------------------------------------------------
	CRect marginRect(720, 720, 720, 1440);
	SetMargins(marginRect);
}

void CSoftCardView::SetTextColor(const COLORREF color)
{
	CHARFORMAT		cf;
	int				editLength;
	CRichEditCtrl	&edit = GetRichEditCtrl();

	ZeroMemory(&cf, sizeof cf);
	cf.cbSize = sizeof cf;

	editLength = edit.GetWindowTextLength();
	edit.SetSel(editLength, editLength);

	//edit.GetSelectionCharFormat(cf);

	cf.dwMask		= CFM_COLOR;
	cf.crTextColor	= color;

	SetCharFormat(cf);

}

#if 0
void CSoftCardView::AddLine(LPCTSTR szTextIn, const COLORREF crNewColor)
{
	CRichEditCtrl	&edit = GetRichEditCtrl();
	int iTotalTextLength = GetWindowTextLength();
	edit.SetSel(iTotalTextLength, iTotalTextLength);
	edit.ReplaceSel(szTextIn);
	int iStartPos = iTotalTextLength;

	CHARFORMAT cf;
	cf.cbSize		= sizeof(CHARFORMAT);
	cf.dwMask		= CFM_COLOR | CFM_UNDERLINE | CFM_BOLD;
	cf.dwEffects	=(unsigned long) ~( CFE_AUTOCOLOR | CFE_UNDERLINE | CFE_BOLD);
	cf.crTextColor	= crNewColor;//RGB(0, 0, 0);

	int iEndPos = edit.GetWindowTextLength();
	edit.SetSel(iStartPos, iEndPos);
	edit.SetSelectionCharFormat(cf);
	edit.HideSelection(TRUE, FALSE);	

	edit.LineScroll(1);
}
#endif

void CSoftCardView::AddLine(LPCTSTR text, const COLORREF color)
{
	CRichEditCtrl	&edit = GetRichEditCtrl();
	int				editLength;
	CString			sss = text;

	sss += _T("\r\n");

	SetTextColor(color);

	editLength = edit.GetWindowTextLength();
	edit.SetSel(editLength, editLength);
	edit.ReplaceSel(sss);
	edit.LineScroll( edit.GetLineCount() );
	GetDocument()->SetModifiedFlag(FALSE);

	SetTextColor(COLOR_BLACK);
}

void CSoftCardView::AddText(LPCTSTR text, const COLORREF color)
{
	CRichEditCtrl	&edit = GetRichEditCtrl();
	int				editLength;
	CString			sss = text;

	SetTextColor(color);

	editLength = edit.GetWindowTextLength();
	edit.SetSel(editLength, editLength);
	edit.ReplaceSel(sss);
	edit.LineScroll( edit.GetLineCount() );
	GetDocument()->SetModifiedFlag(FALSE);

	SetTextColor(COLOR_BLACK);
}

void CSoftCardView::OnTimer(UINT nIDEvent) 
{

	switch ( nIDEvent )
	{
		case TIMER_ATR:
		{
			m_eCardState = STATE_CMD;
			KillTimer(TIMER_ATR);
			CRichEditCtrl	&edit = GetRichEditCtrl();
			int				editLength;

			editLength = edit.GetWindowTextLength();
			edit.SetSel(editLength, editLength);
			if ( !AtStartOfLine() )
			{
				AddLine(_T(""), COLOR_BLACK);
			}
		}
		break;

		default: break;
	}

	CRichEditView::OnTimer(nIDEvent);
}

bool CSoftCardView::AtStartOfLine()
{
	if ( GetTextLength() == 0 )
	{
		return true;
	}
	else
	{
		CRichEditCtrl	&edit = GetRichEditCtrl();
		int				editLength;
		editLength = edit.GetWindowTextLength();
		edit.SetSel(editLength, editLength);
		int lineLength = edit.LineLength(-1);
		return lineLength == 0;
	}

}

static CString GetCmdString(U8 ins)
{
	CString sss;
	switch ( ins )
	{
		case INS_SUBMIT_CODE:	sss = _T("SUBMIT CODE"); break;
		case INS_SELECT_FILE:	sss = _T("SELECT FILE"); break;
		case INS_READ_RECORD:	sss = _T("READ RECORD"); break;
		case INS_WRITE_RECORD:	sss = _T("WRITE RECORD"); break;
		case INS_START_SESSION:	sss = _T("START SESSION"); break;
		case INS_AUTHENTICATE:	sss = _T("AUTHENTICATE"); break;
		case INS_GET_RESPONSE:	sss = _T("GET RESPONSE"); break;
		default:				sss = _T("UNKNOWN"); break;
	}

	return sss;
}

void CSoftCardView::OutputCmd(unsigned char *buf)
{
	CString sss;
	CString ttt;

	if ( !AtStartOfLine() )
	{
		AddLine(_T(""), COLOR_BLACK);
	}

	AddText(_T("COMMAND "), COLOR_HEADER);

	CString cmd = GetCmdString(buf[1]);
	AddLine(cmd, COLOR_CMD_NAME);

	for ( int ii = 0; ii < 5; ii++ )
	{
		ttt.Format(_T("%02X "), (int)buf[ii]);
		sss += ttt;
	}

	AddText(sss, COLOR_CMD);
}

void CSoftCardView::OutputProcedureByte(unsigned char cc)
{
	CString sss;
	sss.Format(_T("%02X "), (int)cc);
	AddText(sss, COLOR_PROCEDURE_BYTE);
}

void CSoftCardView::OutputCmdData(unsigned char *buf, int length)
{
	CString sss;
	CString ttt;

	for ( int ii = 0; ii < length; ii++ )
	{
		if ( m_bASCII )
		{
			if ( buf[ii] >= ' ' && buf[ii] <= 'z' )
			{
				ttt.Format(_T("%c"), buf[ii]);
			}
			else
			{
				ttt = _T('.');
			}
		}
		else
		{
			ttt.Format(_T("%02X "), (int)buf[ii]);
		}
		sss += ttt;
	}

	AddText(sss, COLOR_DATA);

}

void CSoftCardView::OutputStatus(unsigned char *buf)
{
	CString sss;
	CString ttt;

	if ( !AtStartOfLine() )
	{
		AddLine(_T(""), COLOR_BLACK);
	}
	AddLine(_T("STATUS"), COLOR_HEADER);
	for ( int ii = 0; ii < 2; ii++ )
	{
		ttt.Format(_T("%02X "), (int)buf[ii]);
		sss += ttt;
	}

	AddText(sss, COLOR_STATUS);
}

void CSoftCardView::OutputNull()
{
	if ( !AtStartOfLine() )
	{
		AddLine(_T(""), COLOR_BLACK);
	}
	AddLine(_T("null byte"), COLOR_NULL_BYTE);
}

void CSoftCardView::UpdateConnectMenu()
{
	CWnd *pMainWnd = AfxGetMainWnd();
	ASSERT(pMainWnd);
	CMenu *pMenu = pMainWnd->GetMenu();
	ASSERT(pMenu);

	//--------------------------------------------------------------------------
	//	Loop through menu names.
	//--------------------------------------------------------------------------
	for ( UINT iPos = 0; iPos < pMenu->GetMenuItemCount(); iPos++ )
	{
		CString name;
		pMenu->GetMenuString(iPos, name, MF_BYPOSITION);
		if ( name.Find(_T("Connect")) >= 0 )
		{
			CMenu* pSubMenu = pMenu->GetSubMenu(iPos);
			UINT numItems = pSubMenu->GetMenuItemCount();

			//------------------------------------------------------------------
			//	Disconnect is item 0, so do not remove it.
			//------------------------------------------------------------------

			for ( UINT ii = 1; ii < numItems; ii++ )
			{
				BOOL success = pSubMenu->RemoveMenu(0, MF_BYPOSITION);
				ASSERT(success);
			}

			for ( size_t nn = 0; nn < m_ports.size(); nn++ )
			{
				CString sss;
				sss.Format(_T("COM%d"), m_ports[nn]);
				pSubMenu->AppendMenu(MF_STRING, IDM_CONNECT_RS232_BASE + nn, sss);
			}
		}
	}
}

void CSoftCardView::OnConnectRS232(UINT nID) 
{
	OnDisconnect();

	size_t idx = nID - IDM_CONNECT_RS232_BASE;
	ASSERT(idx < m_ports.size());
	m_iPort = m_ports[idx];

	InitPort();
	OnCardNew();
}

void CSoftCardView::OnDisconnect() 
{
	if ( m_serial.IsOpen() )
	{
		m_serial.Close();
	}
	
	CMainFrame *pFrame = dynamic_cast<CMainFrame *>(AfxGetMainWnd());
	pFrame->SetCom(0, m_iBaud);
	pFrame->SetConnection(_T("Disconnected"));

}

void CSoftCardView::OnUpdateDisconnect(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_serial.IsOpen());		
}

void CSoftCardView::OnClear() 
{
	GetRichEditCtrl().SetWindowText(_T(""));	
}

void CSoftCardView::OnUpdateClear(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(GetTextLength() > 0);	
}

void CSoftCardView::OnGotoTop() 
{
	int nFirstVisible = GetRichEditCtrl().GetFirstVisibleLine();

	// Scroll the rich edit control so that the first visible line
	// is the first line of text.
	if (nFirstVisible > 0)
	{
	   GetRichEditCtrl().LineScroll(-nFirstVisible, 0);
	}
	
}

void CSoftCardView::OnUpdateGotoTop(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((GetTextLength() > 0) && (GetRichEditCtrl().GetFirstVisibleLine() > 0));	
}

void CSoftCardView::OnUpdateConnectRs232(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((m_ports.size() > 0));	
}

void CSoftCardView::OnBaud9600() 
{
	m_iBaud = 9600;
	UpdateBaudMenu();
}

void CSoftCardView::OnUpdateBaud9600(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);		
}

void CSoftCardView::OnBaud19200() 
{
	m_iBaud = 19200;
	UpdateBaudMenu();
}

void CSoftCardView::OnUpdateBaud19200(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);		
}

void CSoftCardView::OnBaud38400() 
{
	m_iBaud = 38400;
	UpdateBaudMenu();
}

void CSoftCardView::OnUpdateBaud38400(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);		
}

void CSoftCardView::OnBaud57600() 
{
	m_iBaud = 57600;
	UpdateBaudMenu();
}

void CSoftCardView::OnUpdateBaud57600(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);		
}

void CSoftCardView::OnBaud115200() 
{
	m_iBaud = 115200;
	UpdateBaudMenu();
}

void CSoftCardView::OnUpdateBaud115200(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);	
}

void CSoftCardView::OnBaud128000() 
{
	m_iBaud = 128000;
	UpdateBaudMenu();
}

void CSoftCardView::OnUpdateBaud128000(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);	
}

void CSoftCardView::OnBaud256000() 
{
	m_iBaud = 256000;
	UpdateBaudMenu();
}

void CSoftCardView::OnUpdateBaud256000(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);	
}

void CSoftCardView::UpdateBaudMenu()
{
	CMainFrame *pFrame = dynamic_cast<CMainFrame *>(AfxGetMainWnd());
	pFrame->SetBaud(m_iBaud);

	if ( m_iBaud == 9600 )
	{
		m_pMenu->CheckMenuItem(IDM_BAUD_9600,   MF_BYCOMMAND | MF_CHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_19200,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_38400,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_57600,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_115200, MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_128000, MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_256000, MF_BYCOMMAND | MF_UNCHECKED);
	}
	else if ( m_iBaud == 19200 )
	{
		m_pMenu->CheckMenuItem(IDM_BAUD_9600,   MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_19200,  MF_BYCOMMAND | MF_CHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_38400,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_57600,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_115200, MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_128000, MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_256000, MF_BYCOMMAND | MF_UNCHECKED);
	}
	else if ( m_iBaud == 38400 )
	{
		m_pMenu->CheckMenuItem(IDM_BAUD_9600,   MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_19200,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_38400,  MF_BYCOMMAND | MF_CHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_57600,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_115200, MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_128000, MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_256000, MF_BYCOMMAND | MF_UNCHECKED);
	}
	else if ( m_iBaud == 57600 )
	{
		m_pMenu->CheckMenuItem(IDM_BAUD_9600,   MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_19200,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_38400,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_57600,  MF_BYCOMMAND | MF_CHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_115200, MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_128000, MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_256000, MF_BYCOMMAND | MF_UNCHECKED);
	}
	else if ( m_iBaud == 115200 )
	{
		m_pMenu->CheckMenuItem(IDM_BAUD_9600,   MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_19200,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_38400,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_57600,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_115200, MF_BYCOMMAND | MF_CHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_128000, MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_256000, MF_BYCOMMAND | MF_UNCHECKED);
	}
	else if ( m_iBaud == 128000 )
	{
		m_pMenu->CheckMenuItem(IDM_BAUD_9600,   MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_19200,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_38400,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_57600,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_115200, MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_128000, MF_BYCOMMAND | MF_CHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_256000, MF_BYCOMMAND | MF_UNCHECKED);
	}
	else if ( m_iBaud == 256000 )
	{
		m_pMenu->CheckMenuItem(IDM_BAUD_9600,   MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_19200,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_38400,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_57600,  MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_115200, MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_128000, MF_BYCOMMAND | MF_UNCHECKED);
		m_pMenu->CheckMenuItem(IDM_BAUD_256000, MF_BYCOMMAND | MF_CHECKED);
	}
}

void CSoftCardView::OnAtr() 
{
	CDlgATR	dlg(&m_acos1);

	m_acos1.GetATR(m_atr.acos1Atr, m_atr.iAcos1AtrLength);
	dlg.m_atr = m_atr;

	int nResponse = dlg.DoModal();

	if ( nResponse == IDOK )
	{
		m_atr = dlg.m_atr;
		m_acos1.SetATR(m_atr.acos1Atr, m_atr.iAcos1AtrLength);
	}

}

void CSoftCardView::OnUpdateAtr(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);	
}

void CSoftCardView::ReadSettings()
{
	CWinApp* pApp = AfxGetApp();

	m_iBaud = pApp->GetProfileInt(s_szSectionCom, s_szBaud, DEFAULT_BAUD_RATE);

	m_atr.bAcos1	= pApp->GetProfileInt(s_szSectionATR, s_szIsAcos1,	TRUE);

	bool gotIt;
	gotIt = MyGetProfileBinary(s_szSectionATR, s_szAcos1ATR, m_atr.acos1Atr, m_atr.iAcos1AtrLength, MAX_ATR_LENGTH);
	if ( !gotIt )
	{
		m_acos1.GetDefaultATR(m_atr.acos1Atr, m_atr.iAcos1AtrLength);
	}

	gotIt = MyGetProfileBinary(s_szSectionATR, s_szOtherATR, m_atr.otherAtr, m_atr.iOtherAtrLength, MAX_ATR_LENGTH);
	if ( !gotIt )
	{
		m_acos1.GetDefaultATR(m_atr.otherAtr, m_atr.iOtherAtrLength);
	}
}

void CSoftCardView::WriteSettings()
{
	CWinApp* pApp = AfxGetApp();

	pApp->WriteProfileInt(s_szSectionCom, s_szBaud, m_iBaud);
	pApp->WriteProfileInt(s_szSectionATR, s_szIsAcos1, m_atr.bAcos1);

	MyWriteProfileBinary(s_szSectionATR, s_szAcos1ATR, m_atr.acos1Atr, m_atr.iAcos1AtrLength);
	MyWriteProfileBinary(s_szSectionATR, s_szOtherATR, m_atr.otherAtr, m_atr.iOtherAtrLength);

}

bool CSoftCardView::MyGetProfileBinary
(
	LPCTSTR	szSection,
	LPCTSTR	szKey,
	U8		*buf,
	UINT	&iLength,
	UINT	iMaxLength
)
{
	UINT			nSize = 0 ;
	LPBYTE			pBytes = NULL;
	bool			gotIt = false;

	iLength = 0;
	if ( AfxGetApp()->GetProfileBinary(szSection, szKey, &pBytes, &nSize ) )
	{
		if ( nSize > 0 )
		{
			if ( nSize > iMaxLength )
			{
				nSize = iMaxLength;
			}

			::CopyMemory(buf, pBytes, nSize);
			iLength = nSize;
			gotIt = true;
		}

		delete [] pBytes;
	}

	return gotIt;
}

void CSoftCardView::MyWriteProfileBinary
(
	LPCTSTR	szSection,
	LPCTSTR	szKey,
	U8		*buf,
	UINT	iLength
)
{
	bool	bNeedWriting = true;
	UINT	nSize = 0 ;
	LPBYTE	pBytes = NULL;

	if ( AfxGetApp()->GetProfileBinary(szSection, szKey, &pBytes, &nSize ) )
	{
		if ( nSize == iLength )
		{
			if ( memcmp(&buf, pBytes, nSize) == 0 )
			{
				bNeedWriting = false;
			}
		}

		delete [] pBytes;
	}

	if ( bNeedWriting )
	{
		AfxGetApp()->WriteProfileBinary
		(
			szSection,
			szKey,
			(LPBYTE)buf,
			iLength
		);
	}
}

void CSoftCardView::OnCardNew() 
{
	m_acos1.NewCard();
}

void CSoftCardView::OnUpdateCardNew(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);	
	
}

void CSoftCardView::OnCardLoad() 
{
	// TODO: Add your command handler code here
	
}

void CSoftCardView::OnUpdateCardLoad(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
	
}

void CSoftCardView::OnClearOnReset() 
{
	m_bClearOnReset = !m_bClearOnReset;
}

void CSoftCardView::OnUpdateClearOnReset(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);		
	pCmdUI->SetCheck(m_bClearOnReset);		
}
