//------------------------------------------------------------------------------
//
// Copyright (c) 2004-2005  Matt Brunk  <info@smartcache.net>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License VERSION 2 as
// published by the Free Software Foundation.  You are not allowed to
// use any other version of the license; unless you got the explicit
// permission from the author to do so.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//------------------------------------------------------------------------------
// SoftCardView.h : interface of the CSoftCardView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_SOFTCARDVIEW_H__8162F500_A71B_4A0F_A585_90CC86473958__INCLUDED_)
#define AFX_SOFTCARDVIEW_H__8162F500_A71B_4A0F_A585_90CC86473958__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SerialMFC.h"
#include "ACOS1.h"


#pragma warning( push, 1 )
#include <vector>
#pragma warning( pop )

class CSoftCardView : public CRichEditView
{
protected: // create from serialization only
	CSoftCardView();
	DECLARE_DYNCREATE(CSoftCardView)

// Attributes
public:
	CSoftCardDoc* GetDocument();

	typedef enum
	{
		STATE_NONE,
		STATE_ATR,
		STATE_CMD,
		STATE_INS,
		STATE_DATA,
		STATE_STATUS,
		STATE_ECHO,
		NUM_STATES
	} CardStateEnum;

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CSoftCardView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual void OnInitialUpdate();
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CSoftCardView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CSoftCardView)
	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnDisconnect();
	afx_msg void OnUpdateDisconnect(CCmdUI* pCmdUI);
	afx_msg void OnClear();
	afx_msg void OnUpdateClear(CCmdUI* pCmdUI);
	afx_msg void OnGotoTop();
	afx_msg void OnUpdateGotoTop(CCmdUI* pCmdUI);
	afx_msg void OnUpdateConnectRs232(CCmdUI* pCmdUI);
	afx_msg void OnBaud9600();
	afx_msg void OnUpdateBaud9600(CCmdUI* pCmdUI);
	afx_msg void OnBaud19200();
	afx_msg void OnUpdateBaud19200(CCmdUI* pCmdUI);
	afx_msg void OnBaud38400();
	afx_msg void OnUpdateBaud38400(CCmdUI* pCmdUI);
	afx_msg void OnBaud57600();
	afx_msg void OnUpdateBaud57600(CCmdUI* pCmdUI);
	afx_msg void OnBaud115200();
	afx_msg void OnUpdateBaud115200(CCmdUI* pCmdUI);
	afx_msg void OnBaud128000();
	afx_msg void OnUpdateBaud128000(CCmdUI* pCmdUI);
	afx_msg void OnBaud256000();
	afx_msg void OnUpdateBaud256000(CCmdUI* pCmdUI);
	afx_msg void OnAtr();
	afx_msg void OnUpdateAtr(CCmdUI* pCmdUI);
	afx_msg void OnCardNew();
	afx_msg void OnUpdateCardNew(CCmdUI* pCmdUI);
	afx_msg void OnCardLoad();
	afx_msg void OnUpdateCardLoad(CCmdUI* pCmdUI);
	afx_msg void OnClearOnReset();
	afx_msg void OnUpdateClearOnReset(CCmdUI* pCmdUI);
	//}}AFX_MSG
	afx_msg LRESULT OnSerialMsg (WPARAM wParam, LPARAM lParam);
	DECLARE_MESSAGE_MAP()

	void		DisplayData(unsigned char *pBuf, DWORD length);
	void		Init();
	LONG		InitPort();
	void		AddLine(LPCTSTR text, const COLORREF color);
	void		AddText(LPCTSTR text, const COLORREF color);
	void		SetTextColor(const COLORREF color);
	bool		AtStartOfLine();
	void		OutputCmd(unsigned char *buf);
	void		OutputProcedureByte(unsigned char cc);
	void		OutputCmdData(unsigned char *buf, int length);
	void		OutputStatus(unsigned char *buf);
	void		OutputNull();
	void		UpdateConnectMenu();
	void		OnConnectRS232(UINT nID);
	void		UpdateBaudMenu();
	void		ReadSettings();
	void		WriteSettings();

	bool		MyGetProfileBinary
	(
		LPCTSTR	szSection,
		LPCTSTR	szKey,
		U8		*buf,
		UINT	&iLength,
		UINT	iMaxLength
	);

	void MyWriteProfileBinary
	(
		LPCTSTR	szSection,
		LPCTSTR	szKey,
		U8		*buf,
		UINT	iLength
	);

	CSerialMFC				m_serial;
	int						m_iLineLength;
	int						m_iPort;
	int						m_iBaud;

	CardStateEnum			m_eCardState;
	unsigned char			m_buf[4096];
	int						m_iIdx;
	bool					m_bASCII;
	std::vector<int>		m_ports;
	bool					m_bInitCalled;
	CMenu					*m_pMenu;
	double					m_lastReceiveTime;
	int						m_iNumWrite;
	CACOS1					m_acos1;
	AtrStruct				m_atr;
	unsigned char			m_cardBuf[256];
	UINT					m_cardBufLength;
	BOOL					m_bClearOnReset;
};

#ifndef _DEBUG  // debug version in SoftCardView.cpp
inline CSoftCardDoc* CSoftCardView::GetDocument()
   { return dynamic_cast<CSoftCardDoc*>(m_pDocument); }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_SOFTCARDVIEW_H__8162F500_A71B_4A0F_A585_90CC86473958__INCLUDED_)
